module OmniAuth
  module Strategies
    class Proxy
      PROXY_PATH = "https://oproxy.herokuapp.com/proxy"

      def initialize(app, _options={})
        @app = app
      end

      def call(env)
        status, headers, body = @app.call(env)

        if ENV["OMNIAUTH_PROXY_ENABLED"].present? && status == 302
          location = headers["Location"]

          if is_google?(location) || is_github?(location) || is_auth0?(location)
            headers["Location"] = proxy_uri(location)
          end
        end

        [status, headers, body]
      end

      def is_google?(location)
        location.match?("https://accounts.google.com/o/oauth2/auth")
      end

      def is_github?(location)
        location.match?("https://github.com/login/oauth/authorize")
      end

      def is_auth0?(location)
        location.match?("auth0.com/authorize")
      end

      def proxy_uri(location)
        encoded_uri = Base64.urlsafe_encode64(location)
        PROXY_PATH + "?uri=" + encoded_uri
      end
    end
  end
end
